package com.yizhi.certificate.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.vo.ApiUserCertificateVo;
import com.yizhi.certificate.application.vo.domain.TrUserCertificateVo;
import com.yizhi.certificate.application.vo.ApiSeachCerticateVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 证书
 *
 * @author Fairyland
 */
@FeignClient(name = "certificate", contextId = "CertificateAppClient")
public interface CertificateAppClient {


    /**
     * 根据学员端用户id查询学员端证书数量
     *
     * @return
     */

    @GetMapping("/certificateApp/getCount")
    Integer getCertificateCount(
            @RequestParam(value = "accountId") Long accountId,
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId
    );


    /**
     * 获取我的证书对象
     *
     * @return
     */

    @GetMapping("/certificateApp/get")
    TrUserCertificateVo getCertificate(@RequestParam(value = "id") Long id);


    /**
     * 获取 app我的证书列表
     *
     * @return
     */

    @GetMapping("/certificateApp/list")
    Page<TrUserCertificateVo> list(@RequestBody ApiSeachCerticateVo apiSeachCerticateVo);


    @GetMapping("/certificateApp/appSerachAppList")
    Page<ApiUserCertificateVo> appSerachAppList(@RequestBody ApiSeachCerticateVo apiSeachCerticateVo);


    @PostMapping("/certificateApp/my/update")
    Boolean updateTrUserCertificate(@RequestBody TrUserCertificateVo trUserCertificate);

    @GetMapping("/certificateApp/view")
    TrUserCertificateVo view(
            @RequestParam(value = "id") Long id,
            @RequestParam(value = "accountId") Long accountId,
            @RequestParam(value = "projectId") Long projectId
    );

    /**
     * 周期内用户得到第一个证书的时间
     * @param accountId
     * @param siteIds
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/certificateApp/first")
    public Map<Long, Date> firstCertificate( @RequestParam("siteIds") List<Long> siteIds,
			@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate);

}
