package com.yizhi.certificate.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.vo.*;
import com.yizhi.certificate.application.vo.domain.CertificateVo;
import com.yizhi.certificate.application.vo.domain.RelationActivitiesVo;
import com.yizhi.certificate.application.vo.domain.TrCertificateTemplateVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * 证书
 *
 * @author Fairyland
 */
@FeignClient(name = "certificate", contextId = "CertificateClient")
public interface CertificateClient {


    /**
     * 证书管理端分页列表
     *
     * @param title     证书标题 模糊查询
     * @param companyId 企业id
     * @param siteId    站点id
     * @param orgIds    管辖区id
     * @param status    证书状态 不传全部 1:已关联 2:未关联 3:草稿
     * @param pageNo    分页页数
     * @param pageSize  分页条数
     * @return Page<Certificate>  证书的分页列表
     */
    @GetMapping("/certificate/searchCertificateList")
    Page<CertificateListVO> searchCertificateList(
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "classifyId", required = false) Long classifyId,
            @RequestParam(value = "currentTime", required = false) String currentTime,
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId,
            @RequestParam(value = "orgIds", required = false) List<Long> orgIds,
            @RequestParam(value = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize

    );

    /**
     * 根据证书名称模糊    查询证书列表（分页）   company_id 项目id 为0的证书
     *
     * @return
     */
    @GetMapping("/certificate/getlist")
    PageCertificateVo getList(
            @RequestBody ApiSeachCerticateVo ApiSeachCerticateVo
    );


    /**
     * 解析证书
     *
     * @return
     */

    @GetMapping("/certificate/analysis")
    TrCertificateTemplateVO analysisCertificate(@RequestParam(value = "url") String url, @RequestParam(value = "fileName") String fileName);

    /**
     * 获得证书模板列表
     *
     * @return
     */

    @GetMapping("/certificate/template")
    List<TrCertificateTemplateVo> getTemplateList();

    /**
     * 获得证书列表
     *
     * @return
     */

    @GetMapping("/certificate/list")
    Page<CertificateVo> getCertificateList(@RequestBody CertificateVoList certificateVoList);


    /**
     * 根据证书对象修改证书
     *
     * @return
     */

    @PostMapping("/certificate/update")
    Boolean updateCertificate(@RequestBody CertificateVo certificate);


    /**
     * 新增证书
     *
     * @return
     */

    @PostMapping("/certificate/save")
    Boolean saveCertificate(@RequestBody CertificateVo certificate);

    /**
     * 根据证书id，选择是否启用，修改证书状态
     *
     * @return
     */

    @PostMapping("/certificate/enable")
    Boolean enableCertificate(@RequestBody CertificateVo certificate);

    /**
     * 根据证书id,查询单个证书信息，返回证书对象
     *
     * @return
     */
    @GetMapping("/certificate/get")
    CertificateListVO getCertificate(@RequestParam(value = "id") Long id);

    /**
     * 根据证书id,复制单个证书信息，数据库新增一条数据，证书信息完全一致但名称修改为“***证书【复制件】：证书状态为“下架
     *
     * @return
     */

    @PostMapping("/certificate/copy")
    Boolean copyCertificate(@RequestBody CertificateVo certificate);


    /**
     * 自定义模板保存
     *
     * @param trCertificateTemplate
     * @return
     */
    @PostMapping("/certificate/template/save")
    TrCertificateTemplateVo saveTemplate(@RequestBody TrCertificateTemplateVo trCertificateTemplate);


    @GetMapping("/certificate/by/new/server")
    public List<Map<String, Object>> getServerByCompanyIdAndIds(@RequestParam("companyId") Long companyId, @RequestParam(name = "ids", required = false) List<Long> ids);

    /**
     * 证书清单导出
     *
     * @param title
     * @param status
     * @param classifyId
     * @param currentTime
     * @return
     */
    @GetMapping("/certificate/searchCertificateList/export")
    String searchCertificateListExport(
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "classifyId", required = false) Long classifyId,
            @RequestParam(value = "currentTime", required = false) String currentTime

    );

    /**
     * 证书关联培训项目/课程
     *
     * @param relateCertificateParam
     * @return
     */
    @PostMapping("/certificate/relateProject")
    Boolean relateProject(@RequestBody RelateCertificateParam relateCertificateParam);

    /**
     * 证书取消关联状态
     *
     * @param bizdType
     * @param certificateId
     * @param bizdId
     * @param tpPlanId
     * @return
     */
    @GetMapping("/certificate/cancelRelate")
    Boolean cancelRelate(@RequestParam("bizdType") Integer bizdType,
                         @RequestParam(value = "certificateId",required = false) Long certificateId,
                         @RequestParam(value = "bizdId",required = false) Long bizdId,
                         @RequestParam(value = "tpPlanId", required = false) Long tpPlanId);

    /**
     * 查看关联活动
     * @param certificateId
     * @return
     */
    @GetMapping("/certificate/getRelationActivities")
    List<RelationActivitiesVo> getRelationActivities(@RequestParam("certificateId") Long certificateId);

    /**
     * 通过证书id获取证书信息map
     * @param certificateIds
     * @return
     */
    @GetMapping("/certificate/getCertificateMap")
    Map<Long, CertificateVo> getCertificateMap(@RequestBody List<Long> certificateIds);

    /**
     * 获取某个活动发放数量
     * @param certificateNumParam
     * @return
     */
    @PostMapping("/certificate/getCertificateNum")
    Integer getCertificateNum(@RequestBody CertificateNumParam certificateNumParam);
}
