package com.yizhi.certificate.application.asynchronousTask;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className CertificateListExport
 * @description TODO
 * @date 2019/7/31
 **/
@Component("certificateListExport1")
public class CertificateListExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(CertificateListExport.class);

    @Autowired
    private ICertificateService certificateService;


    @Override
    protected Map execute(Map<String, Object> map) {


        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        String title = (String) map.get("title");
        Integer status = (Integer) map.get("status");
        Long classifyId = (Long) map.get("classifyId");
        String currentTime = (String) map.get("currentTime");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);


        List<CertificateListVO> list = null;
        //获取导出数据
        try {
            Page<CertificateListVO> page = certificateService.searchCertificateList(title, classifyId, res.getCompanyId(),
                    res.getSiteId(), null, status, 1, Integer.MAX_VALUE, currentTime, res);
            list = page.getRecords();
            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        //生成excel,上传阿里云返回url
        try {

            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("证书清单获取");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            //填写表头名
            cell.setCellValue("证书清单获取数据");

            //表头格式
            cell.setCellStyle(style);

            row = sheet.createRow(1);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("证书编号");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("证书名称");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("发证机构");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("创建人");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("创建时间");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("关联活动数");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("获证人数");

            cell = row.createCell(7);
            cell.setCellStyle(style);
            cell.setCellValue("状态");






            //组装数据
            if (!CollectionUtils.isEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    CertificateListVO certificateListVO = list.get(i);

                    row = sheet.createRow(i + 2);

                    row.createCell(0).setCellValue(certificateListVO.getId().toString());

                    row.createCell(1).setCellValue(certificateListVO.getTitle());

                    row.createCell(2).setCellValue(certificateListVO.getIssuer());

                    row.createCell(3).setCellValue(certificateListVO.getCreateByName());

                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");
                    row.createCell(4).setCellValue(sdf.format(certificateListVO.getCreateTime()));

                    row.createCell(5).setCellValue(certificateListVO.getRelationNum());

                    row.createCell(6).setCellValue(certificateListVO.getGrantNum());

                    if (certificateListVO.getStatus()==0){
                        row.createCell(7).setCellValue("下架");
                    }else if (certificateListVO.getStatus()==1){
                        row.createCell(7).setCellValue("上架");
                    }else if (certificateListVO.getStatus()==2){
                        row.createCell(7).setCellValue("草稿");
                    }


                }
            }


            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            String fileName = "证书清单获取数据" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "证书清单数据导出写入过程中发生错误");
                LOGGER.error("证书清单数据导出写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "证书清单数据导出下载发生错误");
            LOGGER.error("证书清单数据导出下载发生错误，请查看日志");
        }

        return null;
    }
}
