package com.yizhi.certificate.application.asynchronousTask;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.service.CertificateUserService;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className CertificateListExport
 * @description TODO
 * @date 2019/7/31
 **/
@Component
public class CertificateStatisticExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(CertificateStatisticExport.class);

    @Autowired
    private CertificateUserService certificateUserService;


    @Override
    protected Map execute(Map<String, Object> map) {


        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        String startDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String certificateName = (String) map.get("certificateName");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);


        List<CertificateListVO> list = null;
        //获取导出数据
        try {
            Page<CertificateListVO> page = certificateUserService.certificateList(startDate, endDate, certificateName, 1, Integer.MAX_VALUE, res);
            list = page.getRecords();
            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        //生成excel,上传阿里云返回url
        try {

            //新建excel对象
            XSSFWorkbook workbook = new XSSFWorkbook();
            XSSFSheet sheet = workbook.createSheet("按证书维度获取统计数据");
            XSSFRow row = sheet.createRow(0);

            CellStyle cellStyle = workbook.createCellStyle();
            XSSFFont font = workbook.createFont();
            font.setBold(true);
            cellStyle.setFont(font);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 4);
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);
            cell.setCellValue("按证书维度获取统计数据");
            cell.setCellStyle(cellStyle);

            row = sheet.createRow(1);
            cell = row.createCell(0);
            cell.setCellValue("统计周期：");
            cell.setCellStyle(cellStyle);

            cra = new CellRangeAddress(1, 1, 1, 4);
            sheet.addMergedRegion(cra);


            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            cell = row.createCell(1);
            cell.setCellValue(null != startDate && null != endDate ? startDate + "~" + endDate : "");
            cell.setCellStyle(cellStyle);

            row = sheet.createRow(2);

            row.createCell(0).setCellValue("证书编码");
            row.createCell(1).setCellValue("证书名称");
            row.createCell(2).setCellValue("发证机构");
            row.createCell(3).setCellValue("获证人数");
            row.createCell(4).setCellValue("状态");

            //组装数据
            if (!CollectionUtils.isEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    CertificateListVO certificateListVO = list.get(i);

                    row = sheet.createRow(i + 3);

                    row.createCell(0).setCellValue(certificateListVO.getId().toString());

                    row.createCell(1).setCellValue(certificateListVO.getTitle());

                    row.createCell(2).setCellValue(certificateListVO.getIssuer());

                    row.createCell(3).setCellValue(certificateListVO.getGrantNum());

                    if (certificateListVO.getStatus() == 0) {
                        row.createCell(4).setCellValue("下架");
                    } else if (certificateListVO.getStatus() == 1) {
                        row.createCell(4).setCellValue("上架");
                    } else if (certificateListVO.getStatus() == 2) {
                        row.createCell(4).setCellValue("草稿");
                    }


                }
            }


            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyyMMddHHmmss");

            String fileName = "证书清单获取数据" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "按证书统计写入过程中发生错误");
                LOGGER.error("按证书统计写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按证书统计下载发生错误");
            LOGGER.error("按证书统计下载发生错误，请查看日志");
        }

        return null;
    }
}
