package com.yizhi.certificate.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.domain.TrUserCertificate;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.service.ITrUserCertificateService;
import com.yizhi.certificate.application.vo.ApiSeachCerticateVo;
import com.yizhi.certificate.application.vo.ApiUserCertificateVo;
import com.yizhi.certificate.application.vo.domain.TrUserCertificateVo;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.util.application.beanutil.BeanCopyListUtil;
import io.swagger.annotations.Api;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Api(tags = "app端证书api", description = "app端证书接口")
@RestController
@RequestMapping("/certificateApp")
public class CertificateAppController {

    private static final Logger logger = LoggerFactory.getLogger(CertificateController.class);

    @Autowired
    private ITrUserCertificateService trUserCertificateService;

    @Autowired
    private ICertificateService certificateService;

    @Autowired
    private ReportClient reportClient;
    /**
     * 根据学员端用户id查询学员端证书数量
     *
     * @return
     */

    @GetMapping("/getCount")
    Integer getCertificateCount(
            @RequestParam(value = "accountId") Long accountId,
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId
    ) {
        return trUserCertificateService.getCertificateCount(accountId,companyId,siteId);
    }

    /**
     * 获取我的证书对象
     *
     * @return
     */

    @GetMapping("/get")
    public TrUserCertificateVo getCertificate(@RequestParam(value = "id") Long id) {
        TrUserCertificate trUserCertificate = trUserCertificateService.selectById(id);
        if (StringUtils.isEmpty(trUserCertificate)) {
            logger.info("*******************************getCertificate="+ JSON.toJSONString(trUserCertificate));
            return null;
        }
        TrUserCertificateVo trUserCertificateVo = new TrUserCertificateVo();
        BeanUtils.copyProperties(trUserCertificate, trUserCertificateVo);
        return trUserCertificateVo;
    }

    /**
     * 获取 app我的证书列表
     *
     * @return
     */

    @GetMapping("/list")
    Page<TrUserCertificateVo> list(@RequestBody ApiSeachCerticateVo apiSeachCerticateVo) {
        Page<TrUserCertificate> trUserCertificates = trUserCertificateService.getMyList(apiSeachCerticateVo.getAccountId(),
                apiSeachCerticateVo.getTitle(),
                apiSeachCerticateVo.getValidityType(),apiSeachCerticateVo.getCompanyId(),apiSeachCerticateVo.getSiteId(),
                apiSeachCerticateVo.getPageNo(), apiSeachCerticateVo.getPageSize());
        Page<TrUserCertificateVo> trUserCertificateVoPage = new Page<>();
        BeanUtils.copyProperties(trUserCertificates, trUserCertificateVoPage);
        trUserCertificateVoPage.setRecords(BeanCopyListUtil.copyListProperties(trUserCertificates.getRecords(), TrUserCertificateVo::new));
        return trUserCertificateVoPage;


    }

    @GetMapping("/appSerachAppList")
    Page<ApiUserCertificateVo> appSerachAppList(@RequestBody ApiSeachCerticateVo apiSeachCerticateVo) {
        Page<ApiUserCertificateVo> page = trUserCertificateService.getMyListSearch(apiSeachCerticateVo.getAccountId(),
                apiSeachCerticateVo.getTitle(), apiSeachCerticateVo.getCompanyId(),apiSeachCerticateVo.getSiteId(),apiSeachCerticateVo.getPageNo(), apiSeachCerticateVo.getPageSize());
        return page;
    }

    @PostMapping("/my/update")
    public Boolean updateTrUserCertificate(@RequestBody TrUserCertificate trUserCertificate) {

        return trUserCertificateService.updateById(trUserCertificate);
    }

    @GetMapping("/view")
    public TrUserCertificateVo view(
            @RequestParam(value = "id") Long id,
            @RequestParam(value = "accountId") Long accountId,
            @RequestParam(value = "projectId") Long projectId
    ){
        TrUserCertificate trUserCertificate = new TrUserCertificate();
        trUserCertificate.setAccountId(accountId);
        trUserCertificate.setCertificateId(id);
        EntityWrapper<TrUserCertificate> entityWrapper = new EntityWrapper<TrUserCertificate>(trUserCertificate);
        TrUserCertificateVo trUserCertificateVo = new TrUserCertificateVo();
        BeanUtils.copyProperties(trUserCertificateService.selectOne(entityWrapper), trUserCertificateVo);
        return trUserCertificateVo;
    }

    @GetMapping("/first")
    public Map<Long, Date> firstCertificate(@RequestParam("siteIds") List<Long> siteIds,
			@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
		return trUserCertificateService.first( siteIds, startDate, endDate);
    }
}
