package com.yizhi.certificate.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.asynchronousTask.CertificateListExport;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.domain.CertificateClassify;
import com.yizhi.certificate.application.domain.RelationActivities;
import com.yizhi.certificate.application.domain.TrCertificateTemplate;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.service.IClassifyService;
import com.yizhi.certificate.application.service.IRelationActivitiesService;
import com.yizhi.certificate.application.service.ITrCertificateTemplateService;
import com.yizhi.certificate.application.vo.*;
import com.yizhi.certificate.application.vo.domain.CertificateVo;
import com.yizhi.certificate.application.vo.domain.RelationActivitiesVo;
import com.yizhi.certificate.application.vo.domain.TrCertificateTemplateVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.util.application.beanutil.BeanCopyListUtil;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;

//import com.fulan.application.util.CertificateZipAnalysisCallBack;

//@Api(tags = "证书接口", description = "certificate Controller")
@RestController
@RequestMapping("/certificate")
public class CertificateController {

    private static final Logger logger = LoggerFactory.getLogger(CertificateController.class);

    @Autowired
    private ICertificateService certificateService;


    @Autowired
    private ITrCertificateTemplateService iTrCertificateTemplateService;

    @Autowired
    @Qualifier("certificate")
    private Certificate certificate;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private CertificateListExport certificateListExport;

    @Autowired
    private IRelationActivitiesService relationActivitiesService;

    @Autowired
    private IClassifyService classifyService;

    /**
     * 证书管理端分页列表
     *
     * @param title     证书标题
     * @param companyId 企业id
     * @param siteId    站点id
     * @param orgIds    管辖区
     * @param status    证书状态  0代表停用，1代表启用
     * @param pageNo    分页页数
     * @param pageSize  分页条数
     * @return Page<Certificate>  证书的分页列表
     */
    @GetMapping("/searchCertificateList")
    public Page<CertificateListVO> searchCertificateList(
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "classifyId", required = false) Long classifyId,
            @RequestParam(value = "currentTime", required = false) String currentTime,
            @RequestParam(value = "companyId") Long companyId,
            @RequestParam(value = "siteId") Long siteId,
            @RequestParam(value = "orgIds", required = false) List<Long> orgIds,
            @RequestParam(value = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize

    ) {
        RequestContext res = ContextHolder.get();
        return certificateService.searchCertificateList(title, classifyId, companyId, siteId, orgIds, status, pageNo, pageSize, currentTime, res);
    }

    /**
     * 根据证书名称模糊 查询证书列表（分页） company_id 项目id 为0的证书
     *
     * @return
     */
    @GetMapping("/getlist")
    public PageCertificateVo getList(@RequestBody ApiSeachCerticateVo ApiSeachCerticateVo

    ) {
        Page<Certificate> page = certificateService.getCertificatePage(ApiSeachCerticateVo.getTitle(),
                ApiSeachCerticateVo.getCompanyId(), ApiSeachCerticateVo.getSiteId(), ApiSeachCerticateVo.getOrgId(),
                ApiSeachCerticateVo.getPageNo(), ApiSeachCerticateVo.getPageSize());

        PageCertificateVo pageCertificateVo = new PageCertificateVo();
        Page<CertificateVo> certificateVoPage = new Page<>();
        BeanUtils.copyProperties(page, certificateVoPage);
        certificateVoPage.setRecords(BeanCopyListUtil.copyListProperties(page.getRecords(), CertificateVo::new));
        pageCertificateVo.setPage(certificateVoPage);

        return pageCertificateVo;
    }

    /**
     * 根据证书对象修改证书
     * 基于对象操作，用于清空circleCount的数值，
     * 解决更新后前端查询会显示出来
     * 使用updateAllColumnById()不要使用updateById(),不然会忽略circleCount字段
     *
     * @return
     */
    @PostMapping("/update")
    public Boolean updateCertificate(@RequestBody Certificate certificate) {

        Certificate certificateUpdate = certificateService.selectById(certificate.getId());
        //certificateUpdate.setCode(certificate.getCode());
        //certificateUpdate.setCreateById(certificate.getCreateById());
        //certificateUpdate.setCreateByName(certificate.getCreateByName());

        certificateUpdate.setTags(certificate.getTags());
        certificateUpdate.setTitle(certificate.getTitle());
        certificateUpdate.setLogourl(certificate.getLogourl());
        certificateUpdate.setIssuer(certificate.getIssuer());
        certificateUpdate.setType(certificate.getType());
        certificateUpdate.setTemplate(certificate.getTemplate());
        certificateUpdate.setTemplateId(certificate.getTemplateId());
        certificateUpdate.setTemplateImage(certificate.getTemplateImage());
        certificateUpdate.setRuleType(certificate.getRuleType());
        certificateUpdate.setClassifyId(certificate.getClassifyId());
        //1为固定有效 2为固定失效期 0或其他为永不过期
        if (certificate.getType().equals(1)) {
            certificateUpdate.setCircleCount(certificate.getCircleCount());
            certificateUpdate.setCircleUnit(certificate.getCircleUnit());
            certificateUpdate.setEndTime(null);
        } else if (certificate.getType().equals(2)) {
            certificateUpdate.setCircleCount(null);
            certificateUpdate.setCircleUnit(null);
            certificateUpdate.setEndTime(certificate.getEndTime());
        } else {
            certificateUpdate.setCircleCount(null);
            certificateUpdate.setCircleUnit(null);
            certificateUpdate.setEndTime(null);
        }

        // 获取请求上下文信息
        RequestContext requestContext = ContextHolder.get();
        // 用于获取当前用户的信息
        certificateUpdate.setUpdateById(requestContext.getAccountId());
        certificateUpdate.setUpdateByName(requestContext.getAccountName());
        certificateUpdate.setUpdateTime(new Date());
        return certificateService.updateAllColumnById(certificateUpdate);
    }

    /**
     * 获得证书列表
     *
     * @return
     */

    @GetMapping("/list")
    public Page<CertificateVo> getCertificateList(@RequestBody CertificateVoList certificateVoList) {
        // 获取证书列表
        Page<CertificateVo> certificateVoPage = new Page<>();
        BeanUtils.copyProperties(certificateService.listCertificate(certificateVoList),certificateVoPage);
        certificateVoPage.setRecords(BeanCopyListUtil.copyListProperties(certificateService.listCertificate(certificateVoList).getRecords(), CertificateVo::new));
        return certificateVoPage;

    }

    /**
     * 获得证书模板列表
     *
     * @return
     */

    @GetMapping("/template")
    public List<TrCertificateTemplateVo> getTemplateList() {
        List<TrCertificateTemplateVo> trCertificateTemplateVos = BeanCopyListUtil.copyListProperties(iTrCertificateTemplateService.getList(), TrCertificateTemplateVo::new);
        return trCertificateTemplateVos;

    }

    /**
     * 新增证书
     *
     * @return
     */

    @PostMapping("/save")
    public Boolean saveCertificate(@RequestBody Certificate certificate) {
        certificate.setId(idGenerator.generate());
        certificate.setCode(idGenerator.generate().toString());
        certificate.setStatus(0);
        //胡涛写
        if (certificate.getType() == 1) {
            Date date = new Date();
            Calendar calendar = Calendar.getInstance();
            //设置时间为当天晚上23:59:59,有效期最后一天24:00后证书失效
            calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
//            calendar.setTime(date);
            int circleCount = certificate.getCircleCount();//获取其周期数值
            int circleUnit = Integer.valueOf(certificate.getCircleUnit());//获取其周期单位（0为年、1为月、2为天）
            switch (circleUnit) {
                case 2:
                    calendar.add(Calendar.DAY_OF_MONTH, circleCount);
                    break;
                case 1:
                    calendar.add(Calendar.MONTH, circleCount);
                    break;
                case 0:
                    calendar.add(Calendar.YEAR, circleCount);
                    break;

            }
            date = calendar.getTime();//重新转为Date
            if (StringUtils.isNotBlank(date.toString())) {
                System.out.println(date);
                certificate.setEndTime(date);
            }
            //
        }
        return certificateService.insert(certificate);
    }

    @ApiOperation(value = "根据证书id，选择是否启用，修改证书状态")
    @PostMapping("/enable")
    public Boolean enableCertificate(@RequestBody Certificate certificate1) {

        // 根据证书Id查询证书对象
        Certificate certificate = certificateService.selectById(certificate1.getId());

        if (certificate != null) {
            if (certificate.getStatus() != null) {
                // 修改证书状态
                if (certificate1.getStatus() == 0) {
                    //如果是下架需要验证是否有关联活动
                    RelationActivities relationActivities = new RelationActivities();
                    relationActivities.setRelationState(1);
                    relationActivities.setCertificateId(certificate1.getId());
                    List<RelationActivities> list = relationActivitiesService.selectList(new EntityWrapper<>(relationActivities));
                    if (!CollectionUtils.isEmpty(list)) {
                        return false;
                    }
                    certificate.setStatus(0);
                } else if (certificate1.getStatus() == 1) {
                    certificate.setStatus(1);
                }
            }
        }

        // 调用修改方法
        certificateService.updateById(certificate);
        return true;
    }

    /**
     * 根据证书id,查询单个证书信息，返回证书对象
     *
     * @return
     */
    @GetMapping("/get")
    public CertificateListVO getCertificate(@RequestParam(value = "id") Long id) {

        Certificate certificate = certificateService.selectById(id);
        CertificateListVO certificateListVO = new CertificateListVO();
        BeanUtils.copyProperties(certificate, certificateListVO);
        if (null != certificate.getClassifyId()) {
            CertificateClassify certificateClassify = classifyService.selectById(certificate.getClassifyId());
            if (certificateClassify.getLevel() == 2) {
                certificateListVO.setOneLevelId(certificateClassify.getParentId());
            } else if (certificateClassify.getLevel() == 3) {
                certificateListVO.setOneLevelId(certificateClassify.getRootId());
                certificateListVO.setTwoLevelId(certificateClassify.getParentId());
            }
        }

        return certificateListVO;

    }

    /**
     * 根据证书id,复制单个证书信息，数据库新增一条数据，证书信息完全一致但名称修改为“***证书【复制件】：证书状态为“下架
     *
     * @return
     */
    @PostMapping("/copy")
    public Boolean copyCertificate(@RequestBody Certificate certificate2) {
        System.out.println(certificate.getId() + "~~~~~~~~~~~~");


        // 根据证书id查询证书信息
        Certificate certificate = certificateService.selectById(certificate2.getId());
        Certificate certificate1 = new Certificate();

        System.out.println(certificate2 + "*********************");

        if (certificate != null) {
            // 将查询到的证书信息复制（添加）到数据库
            certificate1.setId(idGenerator.generate());
            certificate1.setCode(idGenerator.generate().toString());
            certificate1.setSiteId(certificate.getSiteId());
            certificate1.setIssuer(certificate.getIssuer());
            certificate1.setTitle(certificate.getTitle() + "【复制件】");
            certificate1.setTemplate(certificate.getTemplate());
            certificate1.setCreateById(ContextHolder.get().getAccountId());
            certificate1.setCreateByName(ContextHolder.get().getAccountName());
            certificate1.setCreateTime(new Date());
            certificate1.setUpdateById(ContextHolder.get().getAccountId());
            certificate1.setUpdateByName(ContextHolder.get().getAccountName());
            certificate1.setUpdateTime(new Date());
            certificate1.setTags(certificate.getTags());
            certificate1.setCompanyId(certificate.getCompanyId());
            certificate1.setOrgId(certificate.getOrgId());
            certificate1.setLogourl(certificate.getLogourl());
            certificate1.setType(certificate.getType());
            certificate1.setCircleCount(certificate.getCircleCount());
            certificate1.setEndTime(certificate.getEndTime());
            certificate1.setStatus(0);
            certificate1.setCircleUnit(certificate.getCircleUnit());
            certificate1.setTemplateId(certificate.getTemplateId());
            certificate1.setTemplateImage(certificate.getTemplateImage());
            certificate1.setRuleType(certificate.getRuleType());
            certificate1.setClassifyId(certificate.getClassifyId());

            // 调用添加方法
            Boolean f = certificateService.insert(certificate1);

            if (f) {
                return f;
            } else {
                return Boolean.FALSE;
            }

        } else {
            return Boolean.FALSE;
        }
    }


    /**
     * 自定义模板保存
     *
     * @param trCertificateTemplate
     * @return
     */
    @PostMapping("/template/save")
    public TrCertificateTemplateVo saveTemplate(@RequestBody TrCertificateTemplate trCertificateTemplate) {
        trCertificateTemplate.setId(idGenerator.generate());
        TrCertificateTemplateVo trCertificateTemplateVo = new TrCertificateTemplateVo();
        iTrCertificateTemplateService.insert(trCertificateTemplate);
        BeanUtils.copyProperties(trCertificateTemplate, trCertificateTemplateVo);
        return trCertificateTemplateVo;
    }


    @GetMapping(value = "/by/new/server")
    public List<Map<String, Object>> getServerByCompanyIdAndIds(@RequestParam("companyId") Long companyId, @RequestParam(value = "ids", required = false) List<Long> ids) {
        return certificateService.getServerByCompanyIdAndIds(companyId, ids);
    }

    @GetMapping("/searchCertificateList/export")
    String searchCertificateListExport(
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "classifyId", required = false) Long classifyId,
            @RequestParam(value = "currentTime", required = false) String currentTime

    ) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = idGenerator.generate();
        String serialNo = "CERTIFICATE-LIST-EXPORT-" + taskId.toString();
        String taskName = "证书清单导出-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("title", title);
        map.put("status", status);
        map.put("classifyId", classifyId);
        map.put("currentTime", currentTime);

        certificateListExport.execute(map, true);
        return serialNo;
    }

    @PostMapping("/relateProject")
    Boolean relateProject(@RequestBody RelateCertificateParam relateCertificateParam) {
        return certificateService.relateProject(relateCertificateParam);
    }

    @GetMapping("/cancelRelate")
    Boolean cancelRelate(@RequestParam("bizdType") Integer bizdType,
                         @RequestParam(value = "certificateId", required = false) Long certificateId,
                         @RequestParam(value = "bizdId", required = false) Long bizdId,
                         @RequestParam(value = "tpPlanId", required = false) Long tpPlanId) {

        RequestContext res = ContextHolder.get();

        RelationActivities relationActivities = new RelationActivities();
        if (null != certificateId) {
            relationActivities.setCertificateId(certificateId);
        }
        relationActivities.setBizdType(bizdType);
        if (null != bizdId) {
            relationActivities.setBizdId(bizdId);
        }
        relationActivities.setRelationState(1);

        //如果关联培训项目,需要计划id
        if (bizdType == 0) {
            if (null != tpPlanId) {
                relationActivities.setTpPlanId(tpPlanId);
            }
        }

        List<RelationActivities> list = relationActivitiesService.selectList(new EntityWrapper<>(relationActivities));
        if (!CollectionUtils.isEmpty(list)) {
            for (RelationActivities relationActivities1 : list) {
                relationActivities1.setUpdateById(res.getAccountId());
                relationActivities1.setUpdateByName(res.getAccountName());
                relationActivities1.setUpdateTime(new Date());
                relationActivities1.setRelationState(0);
            }
        } else {
            return true;
        }

        relationActivitiesService.updateBatchById(list);
        return true;
    }

    @GetMapping("/getRelationActivities")
    List<RelationActivitiesVo> getRelationActivities(@RequestParam("certificateId") Long certificateId) {
        List<RelationActivitiesVo> relationActivitiesVos = BeanCopyListUtil.copyListProperties(relationActivitiesService.getRelationActivities(certificateId), RelationActivitiesVo::new);
        return relationActivitiesVos;
    }

    @GetMapping("/getCertificateMap")
    Map<Long, Certificate> getCertificateMap(@RequestBody List<Long> certificateIds) {
        return certificateService.getCertificateMap(certificateIds);
    }

    @PostMapping("/getCertificateNum")
    Integer getCertificateNum(@RequestBody CertificateNumParam certificateNumParam) {
        return certificateService.getCertificateNum(certificateNumParam);
    }
}
