package com.yizhi.certificate.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.domain.TrUserCertificate;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.service.ITrUserCertificateService;
import com.yizhi.certificate.application.util.CertificateCodeLock;
import com.yizhi.certificate.application.vo.ApiSeachCerticateVo;
import com.yizhi.certificate.application.vo.PushCertificateParam;
import com.yizhi.certificate.application.vo.domain.TrUserCertificateVo;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.beanutil.BeanCopyListUtil;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.TrainingProjectEvent;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

@Api(tags = "PC端证书api", description = "PC端证书接口")
@RestController
@RequestMapping("/certificatePc")
public class CertificateStuPcController {

    private static final Logger logger = LoggerFactory.getLogger(CertificateController.class);

    @Autowired
    private ITrUserCertificateService trUserCertificateService;

    @Autowired
    ICertificateService iCertificateService;

    @Autowired
    CertificateCodeLock lock;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IdGenerator idGenerator;

    /**
     * 获取 PC端  我的证书列表
     *
     * @return
     */

    @GetMapping("/list")
    Page<TrUserCertificateVo> list(@RequestBody ApiSeachCerticateVo apiSeachCerticateVo) {
        Page<TrUserCertificate> trUserCertificates = trUserCertificateService.getMyList(apiSeachCerticateVo.getAccountId(),
                apiSeachCerticateVo.getTitle(),
                apiSeachCerticateVo.getValidityType(), apiSeachCerticateVo.getCompanyId(), apiSeachCerticateVo.getSiteId(),
                apiSeachCerticateVo.getPageNo(), apiSeachCerticateVo.getPageSize());
        Page<TrUserCertificateVo> trUserCertificateVoPage = new Page<>();
        BeanUtils.copyProperties(trUserCertificates, trUserCertificateVoPage);
        trUserCertificateVoPage.setRecords(BeanCopyListUtil.copyListProperties(trUserCertificates.getRecords(), TrUserCertificateVo::new));
        return trUserCertificateVoPage;
    }

//    /**
//     *   根据学员证书中间表id 获取图片和证书名称( PC端 )
//     *
//     * @param id
//     * @return
//     */
//    @GetMapping("/imageView")
//    public TrUserCertificatePcVo view(
//            @RequestParam(value = "id") Long id,
//            @RequestParam(value = "accountId") Long accountId
//    ){
//        TrUserCertificate trUserCertificate = new TrUserCertificate();
//        trUserCertificate.setAccountId(accountId);
//        trUserCertificate.setCertificateId(id);
//        EntityWrapper<TrUserCertificate> entityWrapper = new EntityWrapper<TrUserCertificate>(trUserCertificate);
//
//        TrUserCertificate tc = trUserCertificateService.selectOne(entityWrapper);
//        TrUserCertificatePcVo vo = new TrUserCertificatePcVo();
//        if (tc != null){
//
//            vo.setCertificateTitle(tc.getCertificateTitle());
//            vo.setCertificateLogourl(tc.getCertificateLogourl());
//            vo.setCertificateId(tc.getCertificateId());
//
//            //相对路径( 去除 http://fulan-test.oss-cn-hangzhou.aliyuncs.com/)
//            String imageStr = tc.getUserImage();
//            String unOss = "http://fulan-test.oss-cn-hangzhou.aliyuncs.com/";
//
//            vo.setUserImage(imageStr);
//            if (!StringUtils.isEmpty(imageStr)){
//                vo.setUnOssImage(imageStr.replace(unOss,""));
//            }
//        }
//
//        return vo;
//    }

    /**
     * 获取我的证书对象 (PC端 证书查看和下载)
     *
     * @return
     */

    @GetMapping("/imageView")
    public TrUserCertificateVo getCertificate(@RequestParam(value = "id") Long id) {
        TrUserCertificate trUserCertificate = trUserCertificateService.selectById(id);
        if (null == trUserCertificate){
            return null;
        }
        Certificate certificate = iCertificateService.selectById(trUserCertificate.getCertificateId());
        if (StringUtils.isBlank(trUserCertificate.getCertificateCode())) {
            lock.addCertificateCode(certificate, trUserCertificate);
            trUserCertificate.setCertificateLogourl(certificate.getTemplateImage());
            trUserCertificate.setIssuer(certificate.getIssuer());
        }
        boolean isOk = trUserCertificateService.updateById(trUserCertificate);
        if (org.springframework.util.StringUtils.isEmpty(trUserCertificate)) {
            logger.info("*******************************getCertificate=" + JSON.toJSONString(trUserCertificate));
            return null;
        }
        //其他业务也可关联证书,发放也要向培训项目发消息
        if (isOk) {
            toProject(trUserCertificate, certificate.getSiteId());
        }
        TrUserCertificateVo trUserCertificateVo = new TrUserCertificateVo();
        BeanUtils.copyProperties(trUserCertificate, trUserCertificateVo);
        return trUserCertificateVo;
    }


    @GetMapping("/view")
    public TrUserCertificateVo view(
            @RequestParam(value = "id") Long id,
            @RequestParam(value = "accountId") Long accountId,
            @RequestParam(value = "projectId") Long projectId,
            @RequestParam(value = "type") Integer type
    ) {
        TrUserCertificate trUserCertificate = new TrUserCertificate();
        trUserCertificate.setAccountId(accountId);
        trUserCertificate.setCertificateId(id);
        EntityWrapper<TrUserCertificate> entityWrapper = new EntityWrapper<TrUserCertificate>(trUserCertificate);
        TrUserCertificate trUserCertificate1 = trUserCertificateService.selectOne(entityWrapper);
        Certificate certificate = iCertificateService.selectById(trUserCertificate1.getCertificateId());
        if (StringUtils.isBlank(trUserCertificate1.getCertificateCode())) {
            lock.addCertificateCode(certificate, trUserCertificate1);
            trUserCertificate1.setCertificateLogourl(certificate.getTemplateImage());
            trUserCertificate1.setIssuer(certificate.getIssuer());
        }
        boolean isOk = trUserCertificateService.updateById(trUserCertificate1);

        if (isOk) {
            toProject(trUserCertificate, certificate.getSiteId());
        }

        TrUserCertificateVo trUserCertificateVo = new TrUserCertificateVo();
        BeanUtils.copyProperties(trUserCertificate1, trUserCertificateVo);
        return trUserCertificateVo;
    }

    /**
     * 给项目发送证书完成消息
     */
    private void toProject(final TrUserCertificate trUserCertificate, final Long siteId) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            public void handle() {
                try {
                    cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                            new EventWrapper<TrainingProjectEvent>(trUserCertificate.getCertificateId(),
                                    TrainingProjectEvent.getInstance(trUserCertificate.getCertificateId(),
                                            TpActivityType.TYPE_CERTIFICATE, trUserCertificate.getAccountId(),
                                            new Date(), siteId, null, Boolean.TRUE)));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    @PostMapping("/pushCertificate")
    public TrUserCertificateVo pushCertificate(@RequestBody PushCertificateParam pushCertificateParam) {
        TrUserCertificateVo trUserCertificateVo = new TrUserCertificateVo();

        RequestContext res = pushCertificateParam.getRes();

        // 验证此人是否已经获得改证书
        TrUserCertificate vailHas = new TrUserCertificate();
        Integer bizdType = pushCertificateParam.getType();

        vailHas.setCertificateId(pushCertificateParam.getCertificateId());
        vailHas.setAccountId(res.getAccountId());
        EntityWrapper<TrUserCertificate> entityWrapper = new EntityWrapper<TrUserCertificate>(vailHas);
        TrUserCertificate tuc = trUserCertificateService.selectOne(entityWrapper);
        if (null != tuc) {
            logger.error("该学员已获得此证书");
            BeanUtils.copyProperties(tuc, trUserCertificateVo);
            return trUserCertificateVo;
        }

        // 验证证书是否存在
        Certificate certificate = iCertificateService.selectById(pushCertificateParam.getCertificateId());
        if (null != certificate) {

            AccountVO accountVO = accountClient.findById(res.getAccountId());
            TrUserCertificate trUserCertificate = new TrUserCertificate();
            trUserCertificate.setId(idGenerator.generate());
            trUserCertificate.setCreateTime(new Date());
            trUserCertificate.setCertificateId(pushCertificateParam.getCertificateId());
            trUserCertificate.setCertificateTitle(certificate.getTitle());
            trUserCertificate.setCertificateLogourl(certificate.getTemplateImage());
            trUserCertificate.setAccountId(res.getAccountId());
            if (null != accountVO) {
                trUserCertificate.setAccountName(accountVO.getName());
                trUserCertificate.setUserName(accountVO.getFullName());
            }
            trUserCertificate.setBizdType(bizdType);

            if (bizdType == 1) {
                //课程
                trUserCertificate.setTpPlanId(pushCertificateParam.getBizdId());
                trUserCertificate.setTpPlanName(pushCertificateParam.getBizdName());
            }


            trUserCertificate.setType(certificate.getType());// 设置过期类型

            lock.addCertificateCode(certificate, trUserCertificate);
            if (StringUtils.isNotBlank(pushCertificateParam.getScore())) {
                trUserCertificate.setScore(pushCertificateParam.getScore());
            } else {
                trUserCertificate.setScore("0");
            }
            trUserCertificate.setIssuer(certificate.getIssuer());
            trUserCertificate.setCompanyId(certificate.getCompanyId());
            trUserCertificate.setSiteId(certificate.getSiteId());
            // AccountVO accountVO =
            // iCertificateService.getAccountVO(accountId);
            // trUserCertificate.setUserName(accountVO.getFullName());
            // trUserCertificate.setAccountName(accountVO.getName());
            // trUserCertificate.setCompanyName(accountVO.getCompanyName());
            if (0 == certificate.getType()) {
                SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                try {
                    trUserCertificate.setEndTime(dft.parse("2038-01-19 01:14:07"));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (1 == certificate.getType()) {

                //设置时间为当天晚上23:59:59,有效期最后一天24:00后证书失效

                Calendar date = Calendar.getInstance();
                date.set(date.get(Calendar.YEAR),
                        date.get(Calendar.MONTH), date.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
                if ("0".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.YEAR, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
                if ("1".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.MONTH, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
                if ("2".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.DAY_OF_MONTH, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
            }
            if (2 == certificate.getType()) {
                trUserCertificate.setEndTime(certificate.getEndTime());
            }
            if (trUserCertificateService.insert(trUserCertificate)) {
                toProject(trUserCertificate, certificate.getSiteId());
            }
            BeanUtils.copyProperties(trUserCertificate, trUserCertificateVo);
            return trUserCertificateVo;
        }
        return null;
    }
}
