package com.yizhi.certificate.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.asynchronousTask.CertificateStatisticExport;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.service.CertificateUserService;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.certificate.application.vo.CertificateUserListVO;
import com.yizhi.certificate.application.vo.TrUserCertificateDetailsVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(tags = "学员证书 管理端", description = "学员证书 管理端")
@RestController
@RequestMapping("/manage/certificate/user")
public class CertificateUserController {

    @Autowired
    CertificateUserService certificateUserService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private CertificateStatisticExport certificateStatisticExport;

    @GetMapping("/rank/list")
    public Page<CertificateUserListVO> userRankList(
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "pageNo", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", required = false) Integer pageSize,
            @RequestParam(name = "siteId", required = false) Long siteId,
            @RequestParam(name = "companyId", required = false) Long companyId,
            @RequestParam(name = "userName", required = false) String userName,
            @RequestParam(name = "orgNameLike", required = false) String orgNameLike,
            @RequestParam(name = "orgIds", required = false) List<Long> orgIds,
            @RequestParam(name = "isAdmin", required = true) boolean isAdmin,
            @RequestParam(name = "accountId", required = true) Long accountId) {
        return certificateUserService.userRankList(startDate, endDate, pageNo, pageSize, siteId,
                companyId, userName, orgNameLike, orgIds, isAdmin, accountId);
    }

    @GetMapping("/details")
    public Page<TrUserCertificateDetailsVo> userDetails(@RequestParam(name = "startDate", required = false) String startDate,
                                                        @RequestParam(name = "endDate", required = false) String endDate,
                                                        @RequestParam(name = "pageNo", required = false) Integer pageNo,
                                                        @RequestParam(name = "pageSize", required = false) Integer pageSize,
                                                        @RequestParam(name = "siteId", required = false) Long siteId,
                                                        @RequestParam(name = "companyId", required = false) Long companyId,
                                                        @RequestParam(name = "userName", required = false) String userName,
                                                        @RequestParam(name = "orgNameLike", required = false) String orgNameLike,
                                                        @RequestParam(name = "orgIds", required = false) List<Long> orgIds,
                                                        @RequestParam(name = "isAdmin", required = true) boolean isAdmin,
                                                        @RequestParam(name = "accountId", required = true) Long accountId) {
        return certificateUserService.userDetails(startDate, endDate, pageNo, pageSize, siteId, companyId, userName, orgNameLike, orgIds, isAdmin, accountId);
    }

    @GetMapping("/certificate/list")
    public Page<CertificateListVO> certificateList(
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "pageNo", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", required = false) Integer pageSize,
            @RequestParam(name = "certificateName", required = false) String certificateName) {
        RequestContext res = ContextHolder.get();
        return certificateUserService.certificateList(startDate, endDate, certificateName, pageNo, pageSize, res);
    }

    @GetMapping("/certificate/export")
    public String certificateListExport(
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "certificateName", required = false) String certificateName) {

        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = idGenerator.generate();
        String serialNo = "CERTIFICATE-STATISTICS-EXPORT-" + taskId.toString();
        String taskName = "按证书统计-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        map.put("certificateName", certificateName);

        certificateStatisticExport.execute(map, true);
        return serialNo;
    }
}
