/**
 * FileName: MyCertificateListener
 * Author:   wenjunlong
 * Date:     2018/6/28 17:33
 * Description: 监听 学员获得证书
 * History:
 * <author>          <time>          <version>          <desc>
 * 作者姓名           修改时间           版本号              描述
 */
package com.yizhi.certificate.application.listener;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.domain.TrUserCertificate;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.service.ITrUserCertificateService;
import com.yizhi.certificate.application.util.CertificateCodeLock;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

/**
 * 〈一句话功能简述〉<br>
 * 〈监听 学员获得证书〉
 *
 * @author wenjunlong
 * @create 2018/6/28
 * @since 1.0.0
 */

@Component
public class MyCertificateListener {

    private static final Logger LOG = LoggerFactory.getLogger(MyCertificateListener.class);

    @Autowired
    ICertificateService iCertificateService;

    @Autowired
    ITrUserCertificateService trUserCertificateService;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private CertificateCodeLock lock;

    @Autowired
    private AccountClient accountClient;

    @RabbitListener(queues = "myCertificate")
    @Transactional
    public void processMyCertifucate(EventWrapper eventWrapper) {

        LOG.info("", eventWrapper);

        Map<String, Object> json = (Map<String, Object>) eventWrapper.getData();
        // 验证必要参数
        Boolean vailParam = json.containsKey("certificateId") && json.containsKey("accountId");

        if (!vailParam) {
            LOG.error("必要参数 证书certificateId，账号accountId 缺失其中一个或多个");
        }

        Long certificateId = null;
        Long accountId = null;
        Long projectId = null;
        String projectName = null;
        Long planId = null;
        String planName = null;
        Integer bizdType = null;
        String courseName = null;
        Long courseId = null;
        try {
            if (null != json.get("certificateId")) {
                certificateId = Long.valueOf(json.get("certificateId").toString());
            }
            if (null != json.get("accountId")) {
                accountId = Long.valueOf(json.get("accountId").toString());
            }

            if (null != json.get("projectId")) {
                projectId = Long.valueOf(json.get("projectId").toString());
            }
            if (null != json.get("projectName")) {
                projectName = json.get("projectName").toString();
            }

            if (null != json.get("bizdType")) {
                bizdType = Integer.valueOf(json.get("bizdType").toString());
            }
            if (null == bizdType) {
                bizdType = 0;
            }

            if (null != json.get("courseName")) {
                courseName = json.get("courseName").toString();
            }

            if (null != json.get("courseId")) {
                courseId = Long.valueOf(json.get("courseId").toString());
            }


            LOG.info("============================================= trPlanId {}", json.get("trPlanId"));

            if (null != json.get("trPlanId")) {
                planId = Long.valueOf(json.get("trPlanId").toString());
            }

            if (null != json.get("trPlanName")) {
                planName = json.get("trPlanName").toString();
            }


        } catch (Exception e) {
            e.printStackTrace();
            LOG.error("参数 证书certificateId(Long)，账号accountId(Long) ，培训项目 projectId(Long) ， "
                    + "培训项目名字projectName(String) 类型转换异常");
        }

        // 验证此人是否已经获得改证书
        TrUserCertificate vailHas = new TrUserCertificate();

        vailHas.setCertificateId(certificateId);
        vailHas.setAccountId(accountId);
        EntityWrapper<TrUserCertificate> entityWrapper = new EntityWrapper<TrUserCertificate>(vailHas);
        TrUserCertificate tuc = trUserCertificateService.selectOne(entityWrapper);
        if (null != tuc) {
            LOG.error("该学员已获得此证书");
            return;
        }

        // 验证证书是否存在
        Certificate certificate = iCertificateService.selectById(certificateId);
        if (null != certificate) {

            AccountVO accountVO = accountClient.findById(accountId);
            TrUserCertificate trUserCertificate = new TrUserCertificate();
            trUserCertificate.setId(idGenerator.generate());
            trUserCertificate.setCreateTime(new Date());
            trUserCertificate.setCertificateId(certificateId);
            trUserCertificate.setCertificateTitle(certificate.getTitle());
            trUserCertificate.setCertificateLogourl(certificate.getTemplateImage());
            trUserCertificate.setAccountId(accountId);
            if (null != accountVO) {
                trUserCertificate.setAccountName(accountVO.getName());
                trUserCertificate.setUserName(accountVO.getFullName());
            }
            trUserCertificate.setBizdType(bizdType);

            if (bizdType == 0) {
                //培训项目
                trUserCertificate.setTrainingProjectId(projectId);
                trUserCertificate.setTrainingProjectName(projectName);
                trUserCertificate.setTpPlanId(planId);
                trUserCertificate.setTpPlanName(planName);
            } else if (bizdType == 1) {
                //课程
                trUserCertificate.setTpPlanId(courseId);
                trUserCertificate.setTpPlanName(courseName);
            }


            trUserCertificate.setType(certificate.getType());// 设置过期类型

            lock.addCertificateCode(certificate, trUserCertificate);
            if (json.containsKey("score")) {
                try {
                    trUserCertificate.setScore(json.get("score").toString());
                } catch (Exception e) {
                }
            } else {
                trUserCertificate.setScore("0");
            }
            trUserCertificate.setIssuer(certificate.getIssuer());
            trUserCertificate.setCompanyId(certificate.getCompanyId());
            trUserCertificate.setSiteId(certificate.getSiteId());
            // AccountVO accountVO =
            // iCertificateService.getAccountVO(accountId);
            // trUserCertificate.setUserName(accountVO.getFullName());
            // trUserCertificate.setAccountName(accountVO.getName());
            // trUserCertificate.setCompanyName(accountVO.getCompanyName());
            if (0 == certificate.getType()) {
                SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                try {
                    trUserCertificate.setEndTime(dft.parse("2038-01-19 01:14:07"));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (1 == certificate.getType()) {

                //设置时间为当天晚上23:59:59,有效期最后一天24:00后证书失效

                Calendar date = Calendar.getInstance();
                date.set(date.get(Calendar.YEAR),
                        date.get(Calendar.MONTH), date.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
                if ("0".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.YEAR, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
                if ("1".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.MONTH, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
                if ("2".equals(certificate.getCircleUnit())) {
                    date.add(Calendar.DAY_OF_MONTH, certificate.getCircleCount());
                    trUserCertificate.setEndTime(date.getTime());
                }
            }
            if (2 == certificate.getType()) {
                trUserCertificate.setEndTime(certificate.getEndTime());
            }
            if (trUserCertificateService.insert(trUserCertificate)) {
                toProject(trUserCertificate, certificate.getSiteId());
            }

        }
    }

    /**
     * 给项目发送证书完成消息
     */
    private void toProject(final TrUserCertificate trUserCertificate, final Long siteId) {
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            public void handle() {
                try {
                    cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                            new EventWrapper<TrainingProjectEvent>(trUserCertificate.getCertificateId(),
                                    TrainingProjectEvent.getInstance(trUserCertificate.getCertificateId(),
                                            TpActivityType.TYPE_CERTIFICATE, trUserCertificate.getAccountId(),
                                            new Date(), siteId, null, Boolean.TRUE)));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

}
