package com.yizhi.certificate.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.vo.CertificateListVO;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.session.RowBounds;

import java.util.List;


/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2018-03-07
 */
public interface CertificateMapper extends BaseMapper<Certificate> {


    /**
     * 根据证书名称模糊    查询证书列表（分页）   company_id 项目id 为0的证书
     *
     * @param title    证书标题
     * @param pageNo   要跳转的页数
     * @param pageSize 每条页数，默认:10
     * @return
     */
    List<Certificate> getCertificatePage(@Param(value = "title") String title, @Param(value = "companyId") Long companyId,
                                         @Param(value = "siteId") Long siteId, @Param(value = "orgIds") List<Long> orgIds, RowBounds rowBounds);


    /**
     * 证书管理端分页列表
     *
     * @param accountIds
     * @param certificateIds 证书ids
     * @return List<Certificate>  证书列表对象
     */
    List<CertificateListVO> searchCertificateList(@Param("accountIds") List<Long> accountIds,
                                                  @Param("certificateIds") List<Long> certificateIds);

    /**
     * 证书管理端分页列表总数
     *
     * @param accountIds
     * @param title      证书标题
     * @param classifyId
     * @param companyId  企业id
     * @param siteId     站点id
     * @param orgIds     管辖区
     * @param status     证书状态  0代表停用，1代表启用
     * @return List<Certificate>  证书列表对象
     */
    Integer searchCertificateListCount(@Param("accountIds") List<Long> accountIds, @Param(value = "title") String title, @Param("classifyId") Long classifyId, @Param(value = "companyId") Long companyId,
                                       @Param(value = "siteId") Long siteId, @Param(value = "orgIds") List<Long> orgIds,
                                       @Param(value = "status") Integer status, @Param("date") String date);

    /**
     * 证书关联培训项目
     *
     * @param bizdId
     * @param bizdName
     * @param certificateIds
     * @param type
     * @return
     */
    Boolean relateProject(@Param("bizdId") Long bizdId,
                          @Param("bizdName") String bizdName,
                          @Param("certificateIds") List<Long> certificateIds,
                          @Param("type") Integer type);

    /**
     * 证书取消关联状态
     *
     * @param certificateId
     * @return
     */
    Boolean cancelRelate(@Param("certificateId") Long certificateId);

    /**
     * 通过证书id获取证书信息map
     *
     * @param certificateIds
     * @return
     */
    List<Certificate> getCertificateMap(@Param("certificateIds") List<Long> certificateIds);

    /**
     * 获取某个活动发放数量
     *
     * @param bizdId
     * @param accountId
     * @param type
     * @param startTime
     * @param endTime
     * @return
     */
    Integer getCertificateNum(@Param("bizdId") Long bizdId,
                              @Param("accountId") Long accountId,
                              @Param("type") Integer type,
                              @Param("startTime") String startTime,
                              @Param("endTime") String endTime);


    /**
     * 证书管理端分页列表id
     *
     * @param title
     * @param classifyId
     * @param companyId
     * @param siteId
     * @param orgIds
     * @param status
     * @param date
     * @param page
     * @return
     */
    List<CertificateListVO> searchCertificateIds(@Param("title") String title,
                                                 @Param("classifyId") Long classifyId,
                                                 @Param("companyId") Long companyId,
                                                 @Param("siteId") Long siteId,
                                                 @Param("orgIds") List<Long> orgIds,
                                                 @Param("status") Integer status,
                                                 @Param("date") String date, Page<CertificateListVO> page);
}
