package com.yizhi.certificate.application.service;


import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.certificate.application.vo.CertificateNumParam;
import com.yizhi.certificate.application.vo.CertificateVoList;
import com.yizhi.certificate.application.vo.RelateCertificateParam;
import com.yizhi.core.application.context.RequestContext;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-07
 */
public interface ICertificateService extends IService<Certificate> {

	/**
	 *  根据证书名称模糊    查询证书列表（分页）   company_id 项目id 为0的证书
 		@param title 证书名称
        * @param pageNo  要跳转的页数
        * @param pageSize 每条页数，默认:10
        * @return
	 */
	Page<Certificate>   getCertificatePage(String title, Long companyId, Long siteId, List<Long> orgIds , Integer pageNo, Integer pageSize );


    /**
     * 获取证书列表
     * @param title
     * @param status
     * @param level
     * @param companyId
     * @param site_id
     * @param org_id
     * @return
     */
    Page<Certificate> listCertificate( CertificateVoList certificateVoList);


	/**
	 * 证书管理端分页列表
	 * @param title  证书标题
	 * @param classifyId
	 * @param companyId  企业id
	 * @param siteId  站点id
	 * @param orgIds  管辖区
	 * @param status  证书状态  0代表停用，1代表启用
	 * @param pageNo  分页页数
	 * @param pageSize 分页条数
	 * @param res
	 * @return Page<Certificate>  证书的分页列表
	 */
	Page<CertificateListVO> searchCertificateList(String title, Long classifyId, Long companyId,
												  Long siteId, List<Long> orgIds,
												  Integer status, Integer pageNo, Integer pageSize, String currentTime, RequestContext res);


	List<Map<String, Object>> getServerByCompanyIdAndIds(Long companyId,List<Long> ids);


	/**
	 * 证书关联培训项目
	 * @param relateCertificateParam
	 * @return
	 */
	Boolean relateProject(RelateCertificateParam relateCertificateParam);

	/**
	 * 证书取消关联状态
	 * @param certificateId
	 * @param projectId
	 * @param tpPlanId
	 * @return
	 */
	Boolean cancelRelate(Long certificateId, Long projectId, Long tpPlanId);

	/**
	 * 通过证书id获取证书信息map
	 * @param certificateIds
	 * @return
	 */
	Map<Long, Certificate> getCertificateMap(List<Long> certificateIds);

	/**
	 * 获取某个活动发放数量
	 * @param certificateNumParam
	 * @return
	 */
    Integer getCertificateNum(CertificateNumParam certificateNumParam);
}
