package com.yizhi.certificate.application.service.impl;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.certificate.application.domain.Certificate;
import com.yizhi.certificate.application.domain.RelationActivities;
import com.yizhi.certificate.application.mapper.CertificateMapper;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.service.ICertificateService;
import com.yizhi.certificate.application.service.IRelationActivitiesService;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.certificate.application.vo.CertificateNumParam;
import com.yizhi.certificate.application.vo.CertificateVoList;
import com.yizhi.certificate.application.vo.RelateCertificateParam;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespInManageVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2018-03-07
 */
@Service
public class CertificateServiceImpl extends ServiceImpl<CertificateMapper, Certificate> implements ICertificateService {
    private Logger logger = LoggerFactory.getLogger(CertificateServiceImpl.class);

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IRelationActivitiesService relationActivitiesService;

    /**
     * 根据证书名称模糊    查询证书列表（分页）   company_id 项目id 为0的证书
     *
     * @param course_id 课程id
     * @param pageNo    要跳转的页数
     * @param pageSize  每条页数，默认:10
     * @return
     */
    @Override
    public Page<Certificate> getCertificatePage(String title, Long companyId, Long siteId, List<Long> orgIds, Integer pageNo, Integer pageSize) {
        HQueryUtil.startHQ(Certificate.class);

        Page<Certificate> page = new Page<Certificate>(pageNo, pageSize);
        page.setRecords(this.baseMapper.getCertificatePage(title, companyId, siteId, orgIds, new RowBounds(page.getOffset(), page.getLimit())));
        return page;
    }


    /**
     * 获取证书查询列表
     *
     * @param title
     * @param status
     * @param level
     * @param companyId
     * @param site_id
     * @param org_id
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<Certificate> listCertificate(CertificateVoList certificateVoList) {
        Page<Certificate> page = new Page<Certificate>(certificateVoList.getPageNo(), certificateVoList.getPageSize());
        Certificate certificate = new Certificate();
        EntityWrapper<Certificate> wrapper = new EntityWrapper<Certificate>(certificate);
        //根据权限判断用户的角色判断查找证书列表的条件
        if (certificateVoList.getLevel() == 1) {
            certificate.setCompanyId(certificateVoList.getCompanyId());
            if (certificateVoList.getStatus() != null) {
                certificate.setStatus(certificateVoList.getStatus());
            }
            if (certificateVoList.getTitle() != null) {
                wrapper.and("title like {0}", "%" + certificateVoList.getTitle() + "%").or("tags like {0}", "%" + certificateVoList.getTitle() + "%").orderBy("create_time", false);
                System.out.println("wrapper--------------- " + wrapper.getSqlSegment());
                try {
                    return this.selectPage(page, wrapper);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error("查找证书列表错误");
                    return null;
                }
            }
            wrapper.orderBy("create_time", false);
            try {
                return this.selectPage(page, wrapper);
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("查找证书列表错误");
                return null;
            }
        } else {
            if (certificateVoList.getLevel() == 2) {
                certificate.setSiteId(certificateVoList.getSiteId());
                if (certificateVoList.getStatus() != null) {
                    certificate.setStatus(certificateVoList.getStatus());
                }
                if (certificateVoList.getTitle() != null) {
                    wrapper.and("title like {0}", "%" + certificateVoList.getTitle() + "%").or("tags like {0}", "%" + certificateVoList.getTitle() + "%)").orderBy("create_time", false);
                    try {
                        return this.selectPage(page, wrapper);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("查找证书列表错误");
                        return null;
                    }
                }
                wrapper.orderBy("create_time", false);
                try {
                    return this.selectPage(page, wrapper);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error("查找证书列表错误");
                    return null;
                }
            } else {
                certificate.setOrgId(certificateVoList.getOrgId());
                if (certificateVoList.getStatus() != null) {
                    certificate.setStatus(certificateVoList.getStatus());
                }
                if (certificateVoList.getTitle() != null) {
                    wrapper.and("title like {0}", "%" + certificateVoList.getTitle() + "%").or("tags like {0}", "%" + certificateVoList.getTitle() + "%)").orderBy("create_time", false);
                    try {
                        return this.selectPage(page, wrapper);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("查找证书列表错误");
                        return null;
                    }
                }


                wrapper.orderBy("create_time", false);
                try {
                    return this.selectPage(page, wrapper);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error("查找证书列表错误");
                    return null;
                }
            }
        }

    }

    @Override
    public Page<CertificateListVO> searchCertificateList(String title, Long classifyId, Long companyId, Long siteId, List<Long> orgIds,
                                                         Integer status, Integer pageNo, Integer pageSize, String currentTime, RequestContext res) {

        String date = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (!StringUtils.isEmpty(currentTime)) {
            Long timeStamp = null;
            try {
                timeStamp = Long.valueOf(currentTime);
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
            date = sdf.format(timeStamp);
        }

        Page<CertificateListVO> page = new Page<>(pageNo, pageSize);

        List<Long> accountIds1 = null;
        //如果是一般管理员加上管辖区
        if (!res.isAdmin()) {
            HQAccountInManageParam hqAccountInManageParam = new HQAccountInManageParam();
            hqAccountInManageParam.setSiteId(res.getSiteId());
            hqAccountInManageParam.setAccountId(res.getAccountId());
            hqAccountInManageParam.setModuleType(AuthzConstant.moduleType.statistics_certificate);
            ReportAccountRespInManageVO respInManageVO = reportClient.getRangeAccountsInManage(hqAccountInManageParam);
            if (null != respInManageVO) {
                //如果不是根目录(根目录不用管)
                if (!respInManageVO.getIsAdmin()) {
                    List<ReportAccountRespVO> accountRespVOS = respInManageVO.getList();
                    accountIds1 = accountRespVOS.stream().map(ReportAccountRespVO::getUserId).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(accountIds1)) {
                        return null;
                    }
                }
            } else {
                return null;
            }
        }


        Integer start = (pageNo - 1) * pageSize;
        HQueryUtil.startHQ(Certificate.class);

        List<CertificateListVO> idList = this.baseMapper.searchCertificateIds(title, classifyId, companyId, siteId, orgIds, status, date, page);
        List<Long> ids = null;
        if (!CollectionUtils.isEmpty(idList)) {
            ids = idList.stream().map(CertificateListVO::getId).collect(Collectors.toList());
        }else {
            return page;
        }

        List<CertificateListVO> list = this.baseMapper.searchCertificateList(accountIds1, ids);

//        Integer count = this.baseMapper.searchCertificateListCount(accountIds1, title, classifyId, companyId, siteId, orgIds, status, date);
//        if (count == 0) {
//            return new Page<>(pageNo, pageSize);
//        }
//        page.setTotal(count);

        if (!CollectionUtils.isEmpty(list)) {
            List<Long> accountIds = new ArrayList<>();
            //获取用户名,培训项目名,课程名
            for (CertificateListVO certificateListVO : list) {
                if (!accountIds.contains(certificateListVO.getCreateById())) {
                    accountIds.add(certificateListVO.getCreateById());
                }
            }

            Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);

            if (!CollectionUtils.isEmpty(accountIds)) {
                ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
                reqVO.setAccountIds(accountIds);
                reqVO.setSiteId(siteId);
                List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(reqVO);
                if (!CollectionUtils.isEmpty(accounts)) {
                    for (ReportAccountRespVO respVO : accounts) {
                        accountMap.put(respVO.getUserId(), respVO);
                    }
                }
            }


            for (CertificateListVO certificateListVO : list) {
                ReportAccountRespVO accountRespVO = accountMap.get(certificateListVO.getCreateById());
                if (null != accountRespVO) {
                    certificateListVO.setCreateByName(accountRespVO.getUserName());
                }
            }

            page.setRecords(list);
        }

        return page;
    }


    @Override
    public List<Map<String, Object>> getServerByCompanyIdAndIds(Long companyId, List<Long> ids) {
        // TODO Auto-generated method stub
        List<Map<String, Object>> listMap = null;
        //查询
        Certificate certificate = new Certificate();
        certificate.setCompanyId(companyId);
        EntityWrapper<Certificate> wrapper = new EntityWrapper<Certificate>(certificate);
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper.in("id", ids);
        }
        List<Certificate> listCertificate = this.selectList(wrapper);
        //循环组装到输出对象
        Map<String, Object> map = null;
        if (!CollectionUtils.isEmpty(listCertificate)) {
            listMap = new ArrayList<Map<String, Object>>();
            for (Certificate c : listCertificate) {
                map = new HashMap<String, Object>();
                map.put("catalog", 6);
                map.put("id", c.getId());
                map.put("name", c.getTitle());
                map.put("logo_url", c.getLogourl());
                listMap.add(map);
            }
        }
        return listMap;
    }

    @Override
    public Boolean relateProject(RelateCertificateParam relateCertificateParam) {

        RequestContext res = ContextHolder.get();

        List<Long> certificateIds = relateCertificateParam.getCertificateIds();
        //除了培训项目关联,保存之前先清除之前的记录
        if (relateCertificateParam.getType() != 0) {
            RelationActivities relationActivities = new RelationActivities();
            relationActivities.setBizdType(relateCertificateParam.getType());
            relationActivities.setRelationState(1);
            relationActivities.setTpPlanId(relateCertificateParam.getBizdId());
            List<RelationActivities> relationActivitiesList = relationActivitiesService.selectList(new EntityWrapper<>(relationActivities));
            if (!CollectionUtils.isEmpty(relationActivitiesList)) {

                for (RelationActivities relationActivities1 : relationActivitiesList) {
                    relationActivities1.setRelationState(0);
                    relationActivities.setUpdateById(res.getAccountId());
                    relationActivities.setUpdateByName(res.getAccountName());
                    relationActivities.setUpdateTime(new Date());
                }
                relationActivitiesService.updateBatchById(relationActivitiesList);
            }
        }

        if (!CollectionUtils.isEmpty(certificateIds)) {
            for (Long certificateId : certificateIds) {
                RelationActivities relationActivities = new RelationActivities();
                relationActivities.setRelationState(1);
                relationActivities.setCertificateId(certificateId);
                relationActivities.setBizdType(relateCertificateParam.getType());

                //如果是关联培训项目,加上计划id
                if (relateCertificateParam.getType() == 0) {
                    relationActivities.setBizdId(relateCertificateParam.getBizdId());
                    relationActivities.setTpPlanId(relateCertificateParam.getTpPlanId());
                } else {
                    relationActivities.setTpPlanId(relateCertificateParam.getBizdId());
                }
                List<RelationActivities> list = relationActivitiesService.selectList(new EntityWrapper<>(relationActivities));
                if (!CollectionUtils.isEmpty(list)) {
                    //如果有关联记录,跳过
                    continue;
                } else {
                    //没有关联记录,插入记录数据
                    relationActivities.setId(idGenerator.generate());
                    relationActivities.setBizdType(relateCertificateParam.getType());

                    if (relateCertificateParam.getType() == 0) {
                        relationActivities.setBizdName(relateCertificateParam.getBizdName());
                        relationActivities.setTpPlanName(relateCertificateParam.getTpPlanName());
                    } else {
                        relationActivities.setTpPlanName(relateCertificateParam.getBizdName());
                    }

                    relationActivities.setCompanyId(res.getCompanyId());
                    relationActivities.setSiteId(res.getSiteId());
                    relationActivities.setCreateById(res.getAccountId());
                    relationActivities.setCreateByName(res.getAccountName());
                    relationActivities.setCreateTime(new Date());
                    relationActivities.setUpdateById(res.getAccountId());
                    relationActivities.setUpdateByName(res.getAccountName());
                    relationActivities.setUpdateTime(new Date());
                    relationActivitiesService.insert(relationActivities);
                }
            }
        }
        return true;
    }

    @Override
    public Boolean cancelRelate(Long certificateId, Long projectId, Long tpPlanId) {
        return this.baseMapper.cancelRelate(certificateId);
    }

    @Override
    public Map<Long, Certificate> getCertificateMap(List<Long> certificateIds) {


        Map<Long, Certificate> map = new HashMap<>(6);
        if (CollectionUtils.isEmpty(certificateIds)) {
            return map;
        }

        List<Certificate> list = this.baseMapper.getCertificateMap(certificateIds);
        if (!CollectionUtils.isEmpty(list)) {
            for (Certificate certificate : list) {
                map.put(certificate.getId(), certificate);
            }
        }
        return map;
    }

    @Override
    public Integer getCertificateNum(CertificateNumParam certificateNumParam) {
        return this.baseMapper.getCertificateNum(certificateNumParam.getBizdId(),
                certificateNumParam.getAccountId(),
                certificateNumParam.getType(),
                certificateNumParam.getStartTime(),
                certificateNumParam.getEndTime());
    }

}
