package com.yizhi.certificate.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.certificate.application.domain.TrUserCertificate;
import com.yizhi.certificate.application.mapper.CertificateUserMapper;
import com.yizhi.certificate.application.service.CertificateUserService;
import com.yizhi.certificate.application.vo.CertificateListVO;
import com.yizhi.certificate.application.vo.CertificateUserListVO;
import com.yizhi.certificate.application.vo.CertificateUserParamVO;
import com.yizhi.certificate.application.vo.TrUserCertificateDetailsVo;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespInManageVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CertificateUserServiceImpl extends
        ServiceImpl<CertificateUserMapper, TrUserCertificate> implements
        CertificateUserService {
    private static final Logger LOGGER = LoggerFactory
            .getLogger(CertificateUserServiceImpl.class);

    @Autowired
    CertificateUserMapper certificateUserMapper;
    @Autowired
    AccountClient accountClient;
    @Autowired
    ReportClient reportClient;

    @Override
    public Page<CertificateUserListVO> userRankList(String startDate,
                                                    String endDate, Integer pageNo, Integer pageSize, Long siteId,
                                                    Long companyId, String userName, String orgNameLike, List<Long> orgIds, boolean isAdmin , Long accountId1) {
        CertificateUserParamVO vo = new CertificateUserParamVO();
        vo.setUserName(userName);
        vo.setOrgNameLike(orgNameLike);
        vo.setCompanyId(companyId);
        vo.setSiteId(siteId);
        vo.setPageNo(pageNo);
        vo.setPageSize(pageSize);
        vo.setOrgIds(orgIds);
        vo.setAccountId(accountId1);

        Page<CertificateUserListVO> page = new Page<CertificateUserListVO>(
                vo.getPageNo(), vo.getPageSize());

        //根据 用户关键字 获取用户id 集合
        List<ReportAccountRespVO> accountVOList = null;
        boolean isGetAccount = true;
        if (isAdmin) {

            //管理员模糊查询
            if (StringUtils.isNotBlank(userName) || StringUtils.isNotBlank(orgNameLike)) {
                isGetAccount = false;
                ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
                reportRangeAccountReqVO.setSiteId(vo.getSiteId());
                if (StringUtils.isNotBlank(userName)) {
                    reportRangeAccountReqVO.setAccountName(userName.trim());
                }
                if (StringUtils.isNotBlank(orgNameLike)) {
                    reportRangeAccountReqVO.setOrgName(orgNameLike.trim());
                }
                accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
                if (CollectionUtils.isEmpty(accountVOList)) {
                    return page;
                }
            }

        } else {
            isGetAccount = false;
            //一般管理员
            HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
            reportRangeAccountReqVO.setAccountId(vo.getAccountId());
            reportRangeAccountReqVO.setSiteId(vo.getSiteId());
            reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_certificate);
            if (StringUtils.isNotBlank(userName)) {
                reportRangeAccountReqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reportRangeAccountReqVO.setOrgName(orgNameLike.trim());
            }
            ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
            if (null != reportAccountRespInManageVO) {
                if (!reportAccountRespInManageVO.getIsAdmin()) {
                    accountVOList = reportAccountRespInManageVO.getList();
                } else {
                    if (StringUtils.isNotBlank(userName) || StringUtils.isNotBlank(orgNameLike)) {
                        isGetAccount = false;
                        ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                        reportRangeAccountReqVO1.setSiteId(vo.getSiteId());
                        if (StringUtils.isNotBlank(userName)) {
                            reportRangeAccountReqVO1.setAccountName(userName.trim());
                        }
                        if (StringUtils.isNotBlank(orgNameLike)) {
                            reportRangeAccountReqVO1.setOrgName(orgNameLike.trim());
                        }
                        accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);
                        if (CollectionUtils.isEmpty(accountVOList)) {
                            return page;
                        }
                    } else {
                        //授权根目录
                        if (StringUtils.isNotBlank(userName) || StringUtils.isNotBlank(orgNameLike)) {
                            isGetAccount = false;
                            ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                            reportRangeAccountReqVO1.setSiteId(vo.getSiteId());
                            if (StringUtils.isNotBlank(userName)) {
                                reportRangeAccountReqVO1.setAccountName(userName.trim());
                            }
                            if (StringUtils.isNotBlank(orgNameLike)) {
                                reportRangeAccountReqVO1.setOrgName(orgNameLike.trim());
                            }
                            accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);
                            if (CollectionUtils.isEmpty(accountVOList)) {
                                return page;
                            }
                        } else {
                            isGetAccount = true;
                        }
                    }
                }
            }
            if (CollectionUtils.isEmpty(accountVOList)) {
                return page;
            }
        }

        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);
        List<Long> accountIds = new ArrayList<>();

        if (!CollectionUtils.isEmpty(accountVOList)) {
            for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                accountIds.add(reportAccountRespVO.getUserId());
                accountMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
            }
        }

        List<CertificateUserListVO> list = null;
        //管理员不带模糊查询或者授权用户管理的根目录,不带用户id筛选
        if (isGetAccount) {
            list = certificateUserMapper.userRankList(
                    vo.getSiteId(), vo.getCompanyId(), null, startDate, endDate, page);
            //根据记录查找对应的用户信息并放入map中,优化性能
            if (!CollectionUtils.isEmpty(list)) {
                for (CertificateUserListVO certificateUserListVO : list) {
                    accountIds.add(certificateUserListVO.getAccountId());
                }
                ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
                reportRangeAccountReqVO.setSiteId(vo.getSiteId());
                reportRangeAccountReqVO.setAccountIds(accountIds);
                accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
                if (!CollectionUtils.isEmpty(accountVOList)) {
                    for (ReportAccountRespVO respVO : accountVOList) {
                        accountMap.put(respVO.getUserId(), respVO);
                    }
                } else {
                    page.setTotal(0);
                    return page;
                }

            }
        } else {
            //其他都带accountIds筛选
            list = certificateUserMapper.userRankList(
                    vo.getSiteId(), vo.getCompanyId(), accountIds, startDate, endDate, page);
        }


        if (!CollectionUtils.isEmpty(list)) {
            for (CertificateUserListVO cvo : list) {
                Long accountId = cvo.getAccountId();
                if (accountId == null) {
                    continue;
                }
                ReportAccountRespVO reportAccountRespVO = accountMap.get(accountId);
                if (reportAccountRespVO != null) {
                    cvo.setUserName(reportAccountRespVO.getUserName());
                    cvo.setName(reportAccountRespVO.getUserFullName());
                    cvo.setOrgName(reportAccountRespVO.getOrgName());
                    cvo.setOrgId(reportAccountRespVO.getOrgId());
                    cvo.setWorkNum(reportAccountRespVO.getWorkNum());
                    cvo.setPosition(reportAccountRespVO.getPosition());
                    cvo.setAccountState(reportAccountRespVO.getStatus() == 1 ? true : false);
                }
            }
            page.setRecords(list);
        }

        return page;
    }

    @Override
    public Page<TrUserCertificateDetailsVo> userDetails(String startDate, String endDate, Integer pageNo, Integer pageSize, Long siteId,
                                                        Long companyId, String userName, String orgNameLike, List<Long> orgIds, boolean isAdmin, Long accountId1) {
        CertificateUserParamVO vo = new CertificateUserParamVO();
        vo.setUserName(userName);
        vo.setOrgNameLike(orgNameLike);
        vo.setCompanyId(companyId);
        vo.setSiteId(siteId);
        vo.setPageNo(pageNo);
        vo.setPageSize(pageSize);
        vo.setOrgIds(orgIds);
        vo.setAccountId(accountId1);

        Page<TrUserCertificateDetailsVo> page = new Page<TrUserCertificateDetailsVo>(
                vo.getPageNo(), vo.getPageSize());

        //根据 用户关键字 获取用户id 集合
        List<ReportAccountRespVO> accountVOList = null;
        if (isAdmin) {
            ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
            reportRangeAccountReqVO.setSiteId(vo.getSiteId());
            if (StringUtils.isNotBlank(userName)) {
                reportRangeAccountReqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reportRangeAccountReqVO.setOrgName(orgNameLike.trim());
            }
            accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
        } else {
            HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
            reportRangeAccountReqVO.setAccountId(vo.getAccountId());
            reportRangeAccountReqVO.setSiteId(vo.getSiteId());
            reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_certificate);
            if (StringUtils.isNotBlank(userName)) {
                reportRangeAccountReqVO.setAccountName(userName.trim());
            }
            if (StringUtils.isNotBlank(orgNameLike)) {
                reportRangeAccountReqVO.setOrgName(orgNameLike.trim());
            }
            ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
            if (null != reportAccountRespInManageVO) {
                if (reportAccountRespInManageVO.getIsAdmin()) {
                    //管辖区为根目录
                    ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                    reportRangeAccountReqVO1.setSiteId(vo.getSiteId());
                    if (StringUtils.isNotBlank(userName)) {
                        reportRangeAccountReqVO1.setAccountName(userName.trim());
                    }
                    if (StringUtils.isNotBlank(orgNameLike)) {
                        reportRangeAccountReqVO1.setOrgName(orgNameLike.trim());
                    }
                    accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);

                } else {
                    accountVOList = reportAccountRespInManageVO.getList();
                }
            }
        }

        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);
        List<Long> accountIds = new ArrayList<>();

        if (!CollectionUtils.isEmpty(accountVOList)) {
            for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                accountIds.add(reportAccountRespVO.getUserId());
                accountMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
            }
        } else {
            return page;
        }
        List<TrUserCertificateDetailsVo> list = certificateUserMapper.userDetailsList(
                vo.getSiteId(), vo.getCompanyId(), accountIds, startDate, endDate, page);
        if (!CollectionUtils.isEmpty(list)) {
            for (TrUserCertificateDetailsVo cvo : list) {
                Long accountId = cvo.getAccountId();
                if (accountId == null) {
                    continue;
                }
                ReportAccountRespVO reportAccountRespVO = accountMap.get(accountId);
                if (reportAccountRespVO != null) {
                    cvo.setUserName(reportAccountRespVO.getUserName());
                    cvo.setFullName(reportAccountRespVO.getUserFullName());
                    cvo.setWorkNum(reportAccountRespVO.getWorkNum());
                    cvo.setPosition(reportAccountRespVO.getPosition());
                    cvo.setOrgName(reportAccountRespVO.getOrgName());
                    cvo.setOrgId(reportAccountRespVO.getOrgId());
                    cvo.setAccountState(reportAccountRespVO.getStatus());
                    cvo.setOrgName(reportAccountRespVO.getOrgName());
                    cvo.setAccountState(reportAccountRespVO.getStatus());
                }
            }
            page.setRecords(list);
        }

        return page;
    }

    @Override
    public Page<CertificateListVO> certificateList(String startDate, String endDate, String certificateName, Integer pageNo, Integer pageSize, RequestContext res) {

        Page<CertificateListVO> page = new Page<>(pageNo, pageSize);

        //根据 用户关键字 获取用户id 集合
        List<Long> accountIds = null;
        if (!res.isAdmin()) {
            HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
            reportRangeAccountReqVO.setAccountId(res.getAccountId());
            reportRangeAccountReqVO.setSiteId(res.getSiteId());
            reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_certificate);
            ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
            if (null != reportAccountRespInManageVO) {
                if (!reportAccountRespInManageVO.getIsAdmin()) {
                    List<ReportAccountRespVO> accountList = reportAccountRespInManageVO.getList();
                    if (!CollectionUtils.isEmpty(accountList)) {
                        accountIds = accountList.stream().map(ReportAccountRespVO::getUserId).collect(Collectors.toList());
                    }
                    if (CollectionUtils.isEmpty(accountIds)) {
                        return page;
                    }
                }
            }
        }


        Integer start = (pageNo - 1) * pageSize;
        List<CertificateListVO> list = certificateUserMapper.certificateList(startDate, endDate, certificateName, start, pageSize, res.getCompanyId(), res.getSiteId(), accountIds);
        Integer total = certificateUserMapper.certificateListCount(startDate, endDate, certificateName, res.getCompanyId(), res.getSiteId(), accountIds);
        page.setTotal(total);
        if (!CollectionUtils.isEmpty(list)) {
            page.setRecords(list);
        }

        return page;
    }
}
