package com.yizhi.certificate.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.domain.CertificateClassify;
import com.yizhi.certificate.application.mapper.ClassifyMapper;
import com.yizhi.certificate.application.service.IClassifyService;
import com.yizhi.certificate.application.vo.CertificateClassifyListVO;
import com.yizhi.certificate.application.vo.CertificateClassifyVO;
import com.yizhi.certificate.application.vo.domain.CertificateClassifyVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.util.application.beanutil.BeanCopyListUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-12-23
 */
@Service
public class ClassifyServiceImpl extends ServiceImpl<ClassifyMapper, CertificateClassify> implements IClassifyService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ClassifyMapper classifyMapper;

    @Override
    public Integer insertClassify(CertificateClassify certificateClassify) {

        RequestContext res = ContextHolder.get();

        if (null != certificateClassify) {

            if (null != certificateClassify.getLevel()) {
                //等级异常
                if (certificateClassify.getLevel() < 1 || certificateClassify.getLevel() > 3){
                    return 2;
                }
            }
            Date date = new Date();

            //验证同级分类下是否重名
            CertificateClassify certificateClassify1 = new CertificateClassify();
            if (certificateClassify.getLevel() == 1) {
                certificateClassify1.setParentId(0L);
            } else {
                certificateClassify1.setParentId(certificateClassify.getParentId());
            }
            certificateClassify1.setCompanyId(res.getCompanyId());
            certificateClassify1.setSiteId(res.getSiteId());
            certificateClassify1.setName(certificateClassify.getName());
            certificateClassify1.setState(1);
            List<CertificateClassify> classifyList = this.selectList(new EntityWrapper<>(certificateClassify1));

            //同级分类下有重名返回
            if (CollectionUtils.isNotEmpty(classifyList)) {
                return 3;
            }


            if (null == certificateClassify.getId()) {
                certificateClassify.setId(idGenerator.generate());
            }
            if (certificateClassify.getLevel() == 1) {
                Integer maxCode = classifyMapper.getMaxCode(res.getCompanyId(), res.getSiteId());
                if (null == maxCode) {
                    maxCode = 999;
                }
                certificateClassify.setParentId(0L);
                certificateClassify.setRootId(0L);
                certificateClassify.setCode(maxCode + 1);
            }

            if (certificateClassify.getLevel() == 2) {
                //二级分类的rootId即为parentId
                certificateClassify.setRootId(certificateClassify.getParentId());
            } else if (certificateClassify.getLevel() == 3) {
                //三级分类的rootId即为上级分类的parentId
                CertificateClassify certificateClassify2 = this.selectById(certificateClassify.getParentId());
                if (null != certificateClassify2) {
                    certificateClassify.setRootId(certificateClassify2.getParentId());
                }
            }

            certificateClassify.setCompanyId(res.getCompanyId());
            certificateClassify.setSiteId(res.getSiteId());
            certificateClassify.setOrgId(res.getOrgId());
            certificateClassify.setCreateById(res.getAccountId());
            certificateClassify.setCreateByName(res.getAccountName());
            certificateClassify.setCreateTime(date);
            certificateClassify.setUpdateById(res.getAccountId());
            certificateClassify.setUpdateByName(res.getAccountName());
            certificateClassify.setUpdateTime(date);

            this.insert(certificateClassify);
            return 1;

        }
        return 2;
    }

    @Override
    public List<CertificateClassifyListVO> getClassifyList() {

        RequestContext res = ContextHolder.get();

        CertificateClassify certificateClassify = new CertificateClassify();
        certificateClassify.setParentId(0L);
        certificateClassify.setState(1);
        certificateClassify.setLevel(1);
        certificateClassify.setCompanyId(res.getCompanyId());
        certificateClassify.setSiteId(res.getSiteId());

        //获取一级分类列表
        List<CertificateClassify> classifyList = this.selectList(new EntityWrapper<>(certificateClassify));

        List<CertificateClassifyListVO> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(classifyList)) {
            list = JSON.parseArray(JSON.toJSONString(classifyList), CertificateClassifyListVO.class);
        } else {
            return list;
        }

        //获取所有二级,三级分类
        CertificateClassify certificateClassify2 = new CertificateClassify();
        certificateClassify2.setState(1);
        certificateClassify2.setCompanyId(res.getCompanyId());
        certificateClassify2.setSiteId(res.getSiteId());
        EntityWrapper<CertificateClassify> entityWrapper = new EntityWrapper<>(certificateClassify2);
        List<Integer> levelList = new ArrayList<>();
        levelList.add(2);
        levelList.add(3);
        entityWrapper.in("level", levelList);
        List<CertificateClassify> list1 = this.selectList(entityWrapper);

        //二级分类map
        Map<Long, List<CertificateClassify>> levelTwoMap = new HashMap<>(16);
        //三级分类map
        Map<Long, List<CertificateClassify>> levelThreeMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list1)) {
            for (CertificateClassify certificateClassify1 : list1) {
                if (certificateClassify1.getLevel() == 2) {
                    List<CertificateClassify> levelTwoList = new ArrayList<>();
                    //获取是否创建对应二级分类下list
                    levelTwoList = levelTwoMap.get(certificateClassify1.getParentId());
                    if (null == levelTwoList) {
                        //没有就新建
                        levelTwoList = new ArrayList<>();
                    }
                    //添加到对应list中放入map中
                    levelTwoList.add(certificateClassify1);
                    levelTwoMap.put(certificateClassify1.getParentId(), levelTwoList);
                } else if (certificateClassify1.getLevel() == 3) {
                    List<CertificateClassify> levelThreeList = new ArrayList<>();
                    //获取是否创建对应三级分类下list
                    levelThreeList = levelThreeMap.get(certificateClassify1.getParentId());
                    if (null == levelThreeList) {
                        //没有就新建
                        levelThreeList = new ArrayList<>();
                    }
                    //添加到对应list中放入map中
                    levelThreeList.add(certificateClassify1);
                    levelThreeMap.put(certificateClassify1.getParentId(), levelThreeList);
                }
            }

            if (CollectionUtils.isNotEmpty(list)) {
                for (CertificateClassifyListVO classifyListVO : list) {

                    //组装codeName
                    String codeName = classifyListVO.getCode().toString() + "--" + classifyListVO.getName();

                    //获取二级分类列表

                    List<CertificateClassify> classifyList1 = levelTwoMap.get(classifyListVO.getId());

                    List<CertificateClassifyVO> classifyVOList = new ArrayList<>();
                    if (CollectionUtils.isNotEmpty(classifyList1)) {
                        classifyVOList = JSON.parseArray(JSON.toJSONString(classifyList1), CertificateClassifyVO.class);
                    }


                    if (CollectionUtils.isNotEmpty(classifyVOList)) {
                        //装配二级分类
                        classifyListVO.setChildren(classifyVOList);

                        for (CertificateClassifyVO classifyVO : classifyVOList) {

                            //获取一级分类
                            List<CertificateClassify> classifyList2 = levelThreeMap.get(classifyVO.getId());

                            if (CollectionUtils.isNotEmpty(classifyList2)) {
                                List<CertificateClassifyVo> certificateClassifyVos = BeanCopyListUtil.copyListProperties(classifyList2, CertificateClassifyVo::new);
                                classifyVO.setChildren(certificateClassifyVos);
                            }

                        }
                    }
                }
            }
        }
        return list;
    }

    @Override
    public boolean updateClassify(CertificateClassify certificateClassify) {

        RequestContext res = ContextHolder.get();

        //验证同级分类下是否重名
        CertificateClassify certificateClassify1 = new CertificateClassify();
        if (certificateClassify.getLevel() == 1) {
            certificateClassify1.setParentId(0L);
        } else {
            certificateClassify1.setParentId(certificateClassify.getParentId());
        }
        certificateClassify1.setCompanyId(res.getCompanyId());
        certificateClassify1.setSiteId(res.getSiteId());
        certificateClassify1.setName(certificateClassify.getName());
        certificateClassify1.setState(1);
        EntityWrapper<CertificateClassify> entityWrapper = new EntityWrapper<>(certificateClassify1);
        entityWrapper.notIn("id", certificateClassify.getId());
        List<CertificateClassify> classifyList = this.selectList(entityWrapper);

        //同级分类下有重名返回
        if (CollectionUtils.isNotEmpty(classifyList)) {
            return false;
        }

        certificateClassify.setUpdateById(res.getAccountId());
        certificateClassify.setUpdateByName(res.getAccountName());
        certificateClassify.setUpdateTime(new Date());
        this.updateById(certificateClassify);

        return true;
    }

    @Override
    public Integer deleteClassify(Long id) {

        CertificateClassify certificateClassify = this.selectById(id);

        RequestContext res = ContextHolder.get();

        //先查找是否被关联
        Integer relationNum = classifyMapper.getRelationNum(id, res.getCompanyId(), res.getSiteId());
        if (relationNum > 0) {
            return 2;
        }

        if (certificateClassify.getLevel() == 1 || certificateClassify.getLevel() == 2) {
            CertificateClassify certificateClassify1 = new CertificateClassify();
            certificateClassify1.setParentId(certificateClassify.getId());
            certificateClassify1.setState(1);
            certificateClassify1.setCompanyId(res.getCompanyId());
            certificateClassify1.setSiteId(res.getSiteId());
            List<CertificateClassify> list = this.selectList(new EntityWrapper<>(certificateClassify1));
            if (CollectionUtils.isNotEmpty(list)) {
                //有子分类,不能删除
                return 2;
            }
        }

        certificateClassify.setState(0);
        certificateClassify.setUpdateTime(new Date());
        certificateClassify.setUpdateById(res.getAccountId());
        certificateClassify.setUpdateByName(res.getAccountName());

        this.updateById(certificateClassify);
        return 1;
    }
}
